/*

	Test of constructing a color-blindness filter in Java, run on the command line.
	
	Args: input image, pairs of: type and output image
	
	Types:
	1: No L
	2: No M
	3: No S
	4: Dog Vision
	5: Brightness
	6: Only L
	7: Only M
	8: Only S	
*/

import java.io.*;
import java.awt.Canvas;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.*;
import javax.imageio.*;

class CBSim extends RGBImageFilter
{
	// Transformation matrices:
	public static final double TransformMatrices[][][] =
	{
		{
			{0.202001295331, 0.991720719265, -0.193722014597},
			{0.163800203026, 0.792663865514, 0.0435359314602},
			{0.00913336570448, -0.0132684300993, 1.00413506439}
		},
		{
			{0.430749076295, 0.717402505462, -0.148151581757},
			{0.336582831043, 0.574447762213, 0.0889694067435},
			{-0.0236572929497, 0.0275635332006, 0.996093759749}
		},
		{
			{0.971710712275, 0.112392320487, -0.0841030327623},
			{0.0219508442818, 0.817739672383, 0.160309483335},
			{-0.0628595877201, 0.880724870686, 0.182134717034}
		},
		{
			{0.316086131719, 0.854894118151, -0.170980249869},
			{0.250572926562, 0.683189199376, 0.0662378740621},
			{-0.00735450321111, 0.00718184676374, 1.00017265645}
		},
		{
			{0.299, 0.587, 0.114},
			{0.299, 0.587, 0.114},
			{0.299, 0.587, 0.114}
		},
		{
			{0.340841450085, 0.580912815482, 0.0782457344332},
			{0.340841450085, 0.580912815482, 0.0782457344332},
			{0.340841450085, 0.580912815482, 0.0782457344332}
		},
		{
			{0.150317227739, 0.722407271325, 0.127275500935},
			{0.150317227739, 0.722407271325, 0.127275500935},
			{0.150317227739, 0.722407271325, 0.127275500935}
		},
		{
			{0.0336717653952, 0.114595364984, 0.851732869621},
			{0.0336717653952, 0.114595364984, 0.851732869621},
			{0.0336717653952, 0.114595364984, 0.851732869621}
		}
	};
	
	public static final String TransformNames[] =
	{
		"No L (Red) - Protanopia",
		"No M (Green) - Deuteranopia",
		"No S (Blue) - Tritanopia",
		"Dog Vision",
		"Gray - Brightness",
		"Gray - L (Red)",
		"Gray - M (Green)",
		"Gray - S (Blue)"
	};
	
	private double TransformMatrix[][];
	
	CBSim(int type)
	{
		TransformMatrix = TransformMatrices[type];
	}
	
	// Process and write an image
	public static void Process(BufferedImage img, int type, String outfile)
	{
		// Construct the filter
		CBSim filter = new CBSim(type);
		
		// Use it -- to use the AWT Toolkit, create a Canvas lightweight dummy object,
		// which can be created in headless mode
		Canvas dummy = new Canvas();
		Image imgf = dummy.createImage(new FilteredImageSource(img.getSource(), filter));
		
		// Create a buffered image from a generic image object
		BufferedImage imgx =
			new BufferedImage(img.getWidth(), img.getHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics2D g = imgx.createGraphics();
		g.drawImage(imgf, 0, 0, null);
		
		try
		{
			ImageIO.write(imgx, "png", new File(outfile));
		}
		catch(IOException ex)
		{
			System.out.println("Error writing image file \"" + outfile + "\": " + ex.toString());
			System.exit(0);
		}
	}

	// Command-line method
	public static void main(String []argv)
	{
		// This beheading is necessary to avoid creating a GUI-app context
		System.setProperty("java.awt.headless", "true");
		
		// Get the file to be processed
		String infile = argv[0];
		BufferedImage img = null;
		try
		{
			img = ImageIO.read(new File(infile));
		}
		catch(Exception ex)
		{
			System.out.println("Error reading image file \"" + infile + "\": " + ex.toString());
			System.exit(0);
		}
		
		// Parse states:
		// 0 -- type
		// 1 -- output file
		int ParseState = 0;
		int type = -1;
		
		for (int i=1; i<argv.length; i++)
		{
			String arg = argv[i];
			switch(ParseState)
			{
			case 0:
				// Get the filter type
				String typestr = arg;
				type = Integer.parseInt(typestr);
				
				ParseState = 1;
				break;
			
			case 1:
				
				if (type >= 1 && type <= 8)
				{
					String outfile = arg;
					
					Process(img,type-1,outfile);
					System.out.println("Processing type " + String.valueOf(type)
						+ ": " + TransformNames[type-1]
						+ ": " + outfile);
				}
				else
				{
					System.out.println("Type out of range: " + String.valueOf(type));
				}
				
				ParseState = 0;
				break;
			}
		}
	}
	
	public int filterRGB(int x, int y, int rgb)
	{
		float r = (rgb >> 16) & 0xff;
		float g = (rgb >> 8) & 0xff;
		float b = (rgb) & 0xff;
		int res = 0xff;
		for (int i=0; i<3; i++)
		{
			double Row[] = TransformMatrix[i];
			double fsum = Row[0]*r + Row[1]*g + Row[2]*b;
			int isum = (int)(fsum + 0.5);
			if (isum < 0) isum = 0;
			else if (isum > 0xff) isum = 0xff;
			res = (res << 8) + isum;
		}
		return res;
	}
}
